//Transmitter/ADC
//Yuchen Zhang yz238
//Richard Zhang rz44
//ECE 4760 Dual-Channel Mobile EMG

//Includes
#include "txrx.h"
#include <avr/interrupt.h>
#include <stdio.h>

//definitions
#define begin {
#define end }
#define data_length 31
#define ADthresh 10 // 10/8ms  so we sample at 10/8ms and then finish every 1 ms

//declarations
volatile char data[data_length]; 
volatile unsigned char packet_count;
volatile unsigned int time;
volatile unsigned int ADtime;
volatile unsigned int ADcount;
volatile unsigned int sampleNum;
volatile unsigned int numChannels;
volatile unsigned char sampleCount;
volatile char Asel ; 		//raw A to D number
volatile char Alooper;
volatile int i;//our iterator variable

//function headers
void generate_data(char count);
void init(void);


//**********************************************************
//timer 0 compare ISR
ISR (TIMER0_COMPA_vect)
begin      
	if (ADtime<=ADthresh){
		ADtime++;
	}
	if (PINC&0x01==1)
	{//if our switch is on the number of channels is 2
		numChannels = 2;
	}
	else//else reset variables...
	{
		numChannels = 1;
	}
end  


//**********************************************************
void generate_data(char count)
begin                   
	data[0] = count;
	data[1] = 26;
	data[2] = 122;
	data[3] = 75;
	data[4] = 52;
end                                                         


//**********************************************************
//initialization
void init()
begin
	DDRA = 0x00;//A is input
	//DDRC = ~0x01;//turns PORTC0 into input
	PORTC = ~0x01; //starting on except for PORTC0
	DDRC = ~0x01;//PORTC0 is input
	//PORTD used for TX.
	//d2 led, d1 TX, d0,d3..7 unused
	DDRD = 0xff;
	PORTD = 0xff;
	//PORTD&=(0<<2); //turn d2 led on as power-on led
                          
  	TIMSK0 = 2;		//turn on timer 0 cmp match ISR 
  	OCR0A =  250;  	//set the compare reg to 250 time ticks
  	TCCR0A = 0b00000010; // turn on clear-on-match
  	TCCR0B = 0b00000010;	// clock prescalar to 8
  	//clk set to interrupt 8 times per millisecond  	

  	txrx_init(1,0,832,1);//TX only - 2400 baud - led on
  	
	//initialize variables
	time=0;
        packet_count=0;
	generate_data(packet_count);
 	sei();
end                                                 



//**********************************************************
int main()
begin

init();


//init the A to D converter 
   //channel zero/ left adj /EXTERNAL Aref
   //!!!CONNECT Aref jumper!!!! Aref and VDD should be connected
   ADMUX = (1<<ADLAR) ;  
   //enable ADC and set prescaler to 1/128*16MHz=125,000
   //and clear interupt enable
   //and start a conversion
   ADCSRA = ((1<<ADEN) | (1<<ADSC) )+ 7 ; 

	Asel = 0;
	Alooper = 0;

ADcount = 1;

sampleNum = 0;
sampleCount = 0;

numChannels = 1;//initially the number of channels we have is 1.  switched by the switch on C.0

while(1)
begin 
		if (ADtime==ADthresh)
		{
			//get the sample  
			sampleNum +=ADCH; //record value for current ADC channel
			sampleCount++;
			
			if (sampleCount == 8)
			{
				data[ADcount] = sampleNum>>3;//We are done sampling so set value of data

				//255 is reserved for telling matlab the beginning of a packet...
				if (data[ADcount] == 255){ data[ADcount]--; }
				//0 is reserved for end string... this doesn't really matter as being at a rail is useless anyway
				if (data[ADcount] == 0  ){ data[ADcount]++; }

				ADcount++;
				sampleCount = 0;
				sampleNum = 0;


				Alooper ++;//loops through channels...extensible to >2 channels
				if (Alooper >= numChannels)
					Alooper = 0;
				
				ADMUX = (1<<ADLAR)|(Alooper); //switch to next one...
			}

			if (ADcount>data_length-1)
			{
				ADcount = 1; //reset our ADcount
				//transmit our packet
				data[0] = packet_count;//Our packet count will be the first block
				tx_me(data, data_length, numChannels);//Transmits our finished packet
				packet_count++;
				PORTD ^=(1<<2);//Transmit so blink light
			}

			ADCSRA |= (1<<ADSC) ; //start another conversion
			ADtime = 0;
		}
end //while
return 0;
end //main
