//Transmit/Receive Data Adapted from Meghan Desai's Project Code

//Includes
#include <avr/interrupt.h>
#include "txrx.h"

//declarations - general
unsigned char txrx_i;

//declarations - RX
unsigned char rx_, decoded_byte, rx_byte, rx_done, rx_length, buffer, rx_started, max_rx_length, rx_led;
char rx_data[MAX_LENGTH_RX];

//declarations - TX
unsigned char tx_data_length, tx_pkt_length, tx_id, tx_byte, tx_ing, tx_pkt_byte, tx_data_byte, tx_led;         
char tx_data[MAX_LENGTH_TX_PKT];
char in_data[MAX_LENGTH_TX_DATA];

//the encoding...
char code[16] = {0b10001011,0b10001101,0b10010011,0b10010101,0b10010110,
0b10011001,0b10011010,0b10011100,0b10100011,0b10100101,0b10100110,0b10101001,
0b10101100,0b10110001,0b10110010,0b10110100};

/****************************************************************************************
** GENERAL FUNCTIONS ********************************************************************
****************************************************************************************/

//**********************************************************
void txrx_init(int tx, int rx, int baud_num, char led) //00-none, 10-tx only, 01-rx only, 11-both
{
	//setup the transmission...
/*
	UCSR0B.3 = tx; //Bit 3 - TXEN0 
	UCSR0B.4 = rx; //Bit 4 - RXEN0
	UCSR0B.5 = tx; //Bit 5 - UDRIE0
	UCSR0B.6 = 0;  //Bit 6 - TXCIE0
	UCSR0B.7 = rx; //Bit 7 - RXCIE0
*/
	UCSR0A |=(1<<U2X0);             /* improve baud rate error by using 2x clk */   
	UCSR0B = (tx<<TXEN0)|(rx<<RXEN0)|(tx<<UDRIE0)|(rx<<RXCIE0);
	UBRR0H = (baud_num>>8);
	UBRR0L =  baud_num;    
	tx_byte=MAX_LENGTH_TX_PKT;
	tx_ing=0;
        tx_led=led;
	rx_done=1;
        rx_led=led;
 }

//**********************************************************
char decodeOne(char msbyte)
{
	txrx_i=0;
        buffer=start_char;
	while(txrx_i<16) {
		if(msbyte==code[txrx_i])
			buffer=txrx_i;
		txrx_i++;
        }
        return buffer;
}

//**********************************************************
char decode(char msbyte, char lsbyte)
{
	txrx_i=0;
	while(txrx_i<16) {
		if(msbyte==code[txrx_i])
			buffer=txrx_i;
		txrx_i++;
	}
	txrx_i=0;
	while(txrx_i<16) {
		if(lsbyte==code[txrx_i]) {
			buffer = (buffer<<4);
			buffer = buffer | txrx_i; 
		}
		txrx_i++;
	}
	return buffer;
}


/****************************************************************************************
** RECEIVE SIDE FUNCTIONS ***************************************************************
****************************************************************************************/

//**********************************************************
//RX Complete ISR
ISR (USART0_RX_vect)
{
        if(rx_done==0)
        {
	        if (rx_byte==0)
        	{
        		rx_data[rx_byte]=UDR0;
	        	
		        if(rx_data[0]==synch_char) rx_started=1;
        		else rx_started=0;
	        	                        
		        if(rx_data[0]==(synch_char & rx_started))
        		{
	        		rx_byte++;
                                if(rx_led==1) PORTD &=(0<<2);
        		}				
	        }
        	else if (rx_byte==1)
	        {
        		rx_data[rx_byte]=UDR0;
	        	if(rx_data[0]==synch_char && rx_data[1]==start_char)
		        {
        			rx_byte++;
       	        	}				
        	}             
	        else
        	{
	        	rx_data[rx_byte]=UDR0;                
		        if (rx_byte==4)
        		{
	        	        rx_length = decode(rx_data[3],rx_data[4]);			
		        }		
        		else if(rx_byte>=max_rx_length || rx_data[rx_byte]==212)
	        	{
		        	if(rx_led==1) PORTD|=(1<<2);
			        rx_done=1;
        		}
	        	rx_byte++;
        	}	                      
        }  
}

//**********************************************************
void rx_reset(char max_rx)
{
	rx_done=0;
	rx_started=0; 
	rx_length=0;
	       rx_byte=0;
	max_rx_length=max_rx;
}

//**********************************************************
char rxdone(void) { return rx_done; }

//**********************************************************
//RECEIVE SIDE ITERATOR
void init_getrx(void) { rx_ = 0; }
         
char get_next_rx_data(void)
{
        if(rx_ <= 2) return rx_data[rx_++];
        else {
                decoded_byte = decode(rx_data[rx_],rx_data[(char)(rx_+1)]);
                rx_ = rx_ + 2;
                return decoded_byte;
        }
}

char rx_empty(void) { return (rx_ < max_rx_length) ? 0 : 1; }


/****************************************************************************************
** TRANSMIT SIDE FUNCTIONS **************************************************************
****************************************************************************************/

//**********************************************************
//UDR Empty ISR
ISR (USART0_UDRE_vect)
{      
	if(tx_byte< tx_pkt_length)
	{
		UDR0=tx_data[tx_byte];
		tx_byte++;
	}				
	if(tx_byte==tx_pkt_length)
	{
                tx_ing=0;
	}
}  

//**********************************************************
void encode(void)
{
	char temp;//stupid temp
        //synchronization.
	tx_data[0] = synch_char; //calibration
	tx_data[1] = synch_char; //calibration
	tx_data[2] = synch_char; //synchronization
	tx_data[3] = synch_char; //synchronization

        //start character
	tx_data[4] = start_char; 

        //1-byte of ID
 	tx_data[5] = code[tx_id]; //Channel Id, static 0 for this unit

        //2-bytes of length
	tx_data[6] = code[tx_data_length>>4]; //length of data to expect.
	
	temp = tx_data_length<<4;
	tx_data[7] = code[temp>>4];
	//tx_data[7] = code[(tx_data_length<<4)>>4]; //length of data to expect.

        //Payload
        tx_pkt_byte=8; tx_data_byte=0; 
        while(tx_data_byte<tx_data_length)
        {
          	tx_data[tx_pkt_byte] = code[in_data[tx_data_byte]>>4];
			
			temp = (in_data[tx_data_byte]<<4);
	        tx_data[(char)(tx_pkt_byte+1)] = code[temp>>4];
			//tx_data[(char)(tx_pkt_byte+1)] = code[(in_data[tx_data_byte]<<4)>>4];
	        
			tx_pkt_byte=tx_pkt_byte+2;
	        tx_data_byte++;
        }

        //End character
	tx_data[tx_pkt_length-1] = end_char;
}             

//**********************************************************
void tx_me(char tx_data[], int length, int id)
{
        if(tx_ing==0)
        {
                txrx_i=0;
                while(txrx_i < length)
                {
                        in_data[txrx_i]=tx_data[txrx_i];
                        txrx_i++;
                }
                tx_id=id;
                tx_data_length=length;
                tx_pkt_length=length*2+9;
                encode();               
                tx_byte=0;
                tx_ing=1;
        }                
} 

//**********************************************************
char txdone(void) { return tx_ing; }
